/*
*********************************************************************************************************
*                                              uC/OS-II
*                                        The Real-Time Kernel
*
*                    Copyright 1992-2021 Silicon Laboratories Inc. www.silabs.com
*
*                                 SPDX-License-Identifier: APACHE-2.0
*
*               This software is subject to an open source license and is distributed by
*                Silicon Laboratories Inc. pursuant to the terms of the Apache License,
*                    Version 2.0 available at www.apache.org/licenses/LICENSE-2.0.
*
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*
*                                    MCS51-STC32 Specific code
*
* Filename : os_cpu_c.c
* Version  : V2.93.01
*修改版本： V1.00
*修改时间： 2023-09-17
*作者：熊仔
*QQ：   616421684
*********************************************************************************************************
*/

//#define   OS_CPU_GLOBALS
#include  <ucos_ii.h>


/*
*********************************************************************************************************
*                                        LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/

#if      (OS_VERSION >= 281) && (OS_TMR_EN > 0)
  static  INT16U  OSTmrCtr;
#endif                                                                  /* #if (OS_VERSION >= 281) && (OS_TMR_EN > 0)               */

/*
*********************************************************************************************************
*                                       OS INITIALIZATION HOOK
*                                            (BEGINNING)
*
* Description: This function is called by OSInit() at the beginning of OSInit().
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts should be disabled during this call.
*********************************************************************************************************
*/
#if OS_CPU_HOOKS_EN > 0 && OS_VERSION > 203
void  OSInitHookBegin (void) 
{
#if OS_VERSION >= 281 && OS_TMR_EN > 0
    OSTmrCtr = 0;
#endif
}
#endif

/*
*********************************************************************************************************
*                                       OS INITIALIZATION HOOK
*                                               (END)
*
* Description: This function is called by OSInit() at the end of OSInit().
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts should be disabled during this call.
*********************************************************************************************************
*/
#if OS_CPU_HOOKS_EN > 0 && OS_VERSION > 203
void  OSInitHookEnd (void) 
{
}
#endif


/*
*********************************************************************************************************
*                                          TASK CREATION HOOK
*
* Description: This function is called when a task is created.
*
* Arguments  : ptcb   is a pointer to the task control block of the task being created.
*
* Note(s)    : 1) Interrupts are disabled during this call.
*********************************************************************************************************
*/
#if OS_CPU_HOOKS_EN > 0
void  OSTaskCreateHook (OS_TCB *ptcb) 
{
#if OS_APP_HOOKS_EN > 0
    App_TaskCreateHook(ptcb);
#else
    if(ptcb);                                                         /* Prevent compiler warning                                 */
#endif
}
#endif


/*
*********************************************************************************************************
*                                           TASK DELETION HOOK
*
* Description: This function is called when a task is deleted.
*
* Arguments  : ptcb   is a pointer to the task control block of the task being deleted.
*
* Note(s)    : 1) Interrupts are disabled during this call.
*********************************************************************************************************
*/
#if OS_CPU_HOOKS_EN > 0
void  OSTaskDelHook (OS_TCB *ptcb) 
{  EA = 1;
#if OS_APP_HOOKS_EN > 0
    App_TaskDelHook(ptcb);
#else
    if(ptcb);                                                         /* Prevent compiler warning                                 */
#endif
}
#endif

/*
*********************************************************************************************************
*                                             IDLE TASK HOOK
*
* Description: This function is called by the idle task.  This hook has been added to allow you to do
*              such things as STOP the CPU to conserve power.
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts are enabled during this call.
*********************************************************************************************************
*/

#if OS_CPU_HOOKS_EN > 0 && OS_VERSION >= 251
void  OSTaskIdleHook (void) 
{
#if OS_APP_HOOKS_EN > 0
    App_TaskIdleHook();
#endif
}
#endif

/*
*********************************************************************************************************
*                                            TASK RETURN HOOK
*
* Description: This function is called if a task accidentally returns.  In other words, a task should
*              either be an infinite loop or delete itself when done.
*
* Arguments  : ptcb      is a pointer to the task control block of the task that is returning.
*
* Note(s)    : none
*********************************************************************************************************
*/

#if OS_CPU_HOOKS_EN > 0u
void  OSTaskReturnHook (OS_TCB  *ptcb) 
{
#if OS_APP_HOOKS_EN > 0u
    App_TaskReturnHook(ptcb);
#else
    if(ptcb);
#endif
}
#endif

/*
*********************************************************************************************************
*                                           STATISTIC TASK HOOK
*
* Description: This function is called every second by uC/OS-II's statistics task.  This allows your
*              application to add functionality to the statistics task.
*
* Arguments  : none
*********************************************************************************************************
*/

#if OS_CPU_HOOKS_EN > 0
void  OSTaskStatHook (void) 
{
#if OS_APP_HOOKS_EN > 0
    App_TaskStatHook();
#endif
}
#endif


/*
**********************************************************************************************************
*                                       INITIALIZE A TASK'S STACK
*
* Description: This function is called by either OSTaskCreate() or OSTaskCreateExt() to initialize the
*              stack frame of the task being created. This function is highly processor specific.
*
* Arguments  : task          is a pointer to the task code
*
*              p_arg         is a pointer to a user supplied data area that will be passed to the task
*                            when the task first executes.
*
*              ptos          is a pointer to the top of stack. It is assumed that 'ptos' points to the
*                            highest valid address on the stack.
*
*              opt           specifies options that can be used to alter the behavior of OSTaskStkInit().
*                            (see uCOS_II.H for OS_TASK_OPT_???).
*
* Returns    : Always returns the location of the new top-of-stack' once the processor registers have
*              been placed on the stack in the proper order.
**********************************************************************************************************
*/

OS_STK  *OSTaskStkInit (void (*task)(void *pd), void *p_arg, OS_STK *ptos, INT16U opt) 
{
    INT32U ulAddress;
    INT8U index;

    if(opt);                                    /* 'opt' is not used, prevent warning */

    ulAddress = ( INT32U ) task;
    *ptos = 0;                                  /* PSW1 */
    ptos++;
    *ptos = ( OS_STK ) ( ulAddress >> 16 );     /* PC[23:16] */
    ptos++;
    *ptos = ( OS_STK ) ( ulAddress );           /* PC[7:0] */
    ptos++;
    *ptos = ( OS_STK ) ( ulAddress >> 8 );      /* PC[15:8] */

    ptos++;
    *ptos = 0;                                  /* R56 */
    ptos++;
    *ptos = 1;                                  /* R57(DPXL) */
    ptos++;
    *ptos = 0;                                  /* R58(DPH) */
    ptos++;
    *ptos = 0;                                  /* R59(DPL) */

    for(index = 0; index < 28; index++)
    {
        ptos++;
        *ptos = 0;
    }

    ptos++;
    ulAddress = ( INT32U ) p_arg;
    *ptos = ( OS_STK ) ( ulAddress >> 24 );     /* R0 */
    ptos++;
    *ptos = ( OS_STK ) ( ulAddress >> 16 );     /* R1 */
    ptos++;
    *ptos = ( OS_STK ) ( ulAddress >> 8 );      /* R2 */
    ptos++;
    *ptos = ( OS_STK ) ( ulAddress );           /* R3 */
    ptos++;
    *ptos = ( OS_STK ) 0;                       /* PSW */

    return ptos;
}

/*
*********************************************************************************************************
*                                           TASK SWITCH HOOK
*
* Description: This function is called when a task switch is performed.  This allows you to perform other
*              operations during a context switch.
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts are disabled during this call.
*              2) It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task that
*                 will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points to the
*                 task being switched out (i.e. the preempted task).
*********************************************************************************************************
*/
#if (OS_CPU_HOOKS_EN > 0) && (OS_TASK_SW_HOOK_EN > 0)
void  OSTaskSwHook (void) 
{
#if OS_APP_HOOKS_EN > 0
    App_TaskSwHook();
#endif
}
#endif

/*
*********************************************************************************************************
*                                           OS_TCBInit() HOOK
*
* Description: This function is called by OS_TCBInit() after setting up most of the TCB.
*
* Arguments  : ptcb    is a pointer to the TCB of the task being created.
*
* Note(s)    : 1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/
#if OS_CPU_HOOKS_EN > 0 && OS_VERSION > 203
void  OSTCBInitHook (OS_TCB *ptcb) 
{
#if OS_APP_HOOKS_EN > 0
    App_TCBInitHook(ptcb);
#else
    if(ptcb);                                                         /* Prevent compiler warning                                 */
#endif
}
#endif


/*
*********************************************************************************************************
*                                               TICK HOOK
*
* Description: This function is called every tick.
*
* Arguments  : none
*
* Note(s)    : 1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/
#if (OS_CPU_HOOKS_EN > 0) && (OS_TIME_TICK_HOOK_EN > 0)
void  OSTimeTickHook (void) 
{
#if OS_APP_HOOKS_EN > 0
    App_TimeTickHook();
#endif

#if OS_VERSION >= 281 && OS_TMR_EN > 0
    OSTmrCtr++;
    if (OSTmrCtr >= (OS_TICKS_PER_SEC / OS_TMR_CFG_TICKS_PER_SEC)) {
        OSTmrCtr = 0;
        OSTmrSignal();
    }
#endif
}
#endif
                                     


#if OS_CRITICAL_METHOD == 3
//JBC指令方式
//OS_CPU_SR  OS_CPU_SR_Save(void)
//{
//    if(_testbit_(EA)){
//        return 0x80;
//    }else{
//        return 0x00;
//    }        
//}
//进一步优化
OS_CPU_SR  OS_CPU_SR_Save(void)
{
    OS_CPU_SR r = 0x80;
    if(!_testbit_(EA)){
        r = 0x00;
    }
    return r;
}

void OS_CPU_SR_Restore(OS_CPU_SR cpu_sr)
{
    IE |= cpu_sr;
}
#endif 

#if OS_CRITICAL_METHOD==4
//JBC指令方式
//加锁/解锁方式，嵌套计数保存在全局变量CriticalNesting之中
INT8U data CriticalNesting = 0;   //养成好习惯初始化为0，虽然data区默认0.
bit _bEA;                         //第一层进入备份EA的值，最后退出的时候，恢复EA

void OSEnterCritical(void) {
    if (_testbit_(EA)) {
        if (CriticalNesting == 0) {
            _bEA = 1;
        }
    } else {
        if (CriticalNesting == 0) {
            _bEA = 0;
        }
    }
    CriticalNesting++;
}

void OSEXitCritical(void) {
    if (CriticalNesting > 0) {
        CriticalNesting--;
        if (CriticalNesting == 0) {
            EA = _bEA;
        }
    }
}

#endif
