;********************************************************************************************************
;
;                                       MCS51-STC8 Specific code
;                                           keil C51 Compiler
;
; Filename : isr.A51
;主意：统一管理中断
;规定整个工程用寄存器组0,允许中断嵌套的中断服务程序不能修改寄存器组。
;
;用户想用其他寄存器组，无非就是想加快中断执行速度。是否有更好的办法呢？
;答案是有的，把需要加速的中断设置成不可嵌套中断就行。进出堆栈不使用PULLALL和POPALL，
;只需要保存ACC，B，DPH，DPL，PSW寄存器就可以。
;可以参考定时器3，演示使用寄存器组1例子
;********************************************************************************************************

$NOMOD51
#include ".\Source\Ports\os_cpu_i.h"

    
        NAME ISR_A    ;模块名
            
;声明引用全局变量和外部子程序
        EXTRN CODE  (_?OSIntEnter)
        EXTRN CODE  (_?OSIntExit)

    
; **** 中断矢量区域 *************

; ==== 定时器1 中断服务程序 =============
        CSEG AT 001BH	;定时器T1中断
        LJMP Timer1_ISR
        
; ==== 定时器3 中断服务程序 =============
        CSEG AT 009BH	;定时器T3中断
        LJMP Timer3_ISR
        




; ==== 定时器1 中断服务程序 =============
;演示允许中断嵌套
EXTRN CODE  (_?Timer1_Handler)
Timer1_ISR:
        CLR EA  ;调用OSIntEnter之前先关中断
SETB 0A0H
        PUSHALL     ;现场保护
        LCALL _?OSIntEnter	;通知内核进入中断     
        SETB EA  ;允许中断嵌套
        
        LCALL _?Timer1_Handler
        
        LCALL _?OSIntExit   ;通知内核退出中断
        POPALL      ;恢复现场   
CLR 0A0H
        RETI
        
        
; ==== 定时器3 中断服务程序 =============
;演示禁止中断嵌套
EXTRN CODE  (_?Timer3_Handler)
Timer3_ISR:
SETB 0A6H;
        CLR EA      ;禁止中断嵌套
        PUSHALL     ;现场保护

        LCALL _?Timer3_Handler
        
        POPALL      ;恢复现场   
CLR    0A6H
        SETB  EA   ;中断函数执行完毕，重新允许中断
        RETI

/*
; ==== 定时器3 中断服务程序 =============
;演示使用寄存器组1，加快中断执行速度，必须禁止中断嵌套。
EXTRN CODE  (_?Timer3_Handler)
Timer3_ISR:
SETB 0A6H
        CLR EA      ;禁止中断嵌套
        PUSH     ACC
        PUSH     B
        PUSH     DPH
        PUSH     DPL
        PUSH     PSW
;根据实际需求对DPS，DPL1，DPH1，P_SW2入栈
;        PUSH DPS
;        PUSH DPL1
;        PUSH DPH1
;        PUSH P_SW2


        MOV     PSW, #(1 SHL 3)   ;选择寄存器组 1     

        LCALL    _?Timer3_Handler

;根据实际需求对DPS，DPL1，DPH1，P_SW2出栈
;        POP  ACC
;        ANL  A,#0x80
;        ANL  P_SW2,#0x7F 
;        ORL  P_SW2,A
;        POP  DPH1
;        POP  DPL1
;        POP  DPS

        POP      PSW
        POP      DPL
        POP      DPH
        POP      B
        POP      ACC
        SETB  EA   ;中断函数执行完毕，重新允许中断
CLR 0A6H
        RETI
*/        
;--------------------
END
;--------------------
